#include "file.h"
#include "constant\constant.h"

using namespace System;
using namespace System::IO;
using namespace System::Text;
using namespace System::Collections::Generic;


namespace TechnoAP { namespace usbmca {

	namespace common
	{
		/// <summary>
		/// Write from binary file
		/// </summary>
		bool file::WriteBinary(String ^savefile, __int64 qwSeek, int nBytes, array<Byte> ^abBuffer)
		{
			FileStream ^fsSave = nullptr;
			bool result = true;

			try {
				// Open the write destination file
				fsSave = File::Open(savefile, FileMode::OpenOrCreate, FileAccess::Write, FileShare::ReadWrite);

				if (qwSeek == mSEEK_END) {
					// Write from the end of the file
					fsSave->Seek(0, SeekOrigin::End);
				}
				else {
					// Relative position from the beginning
					fsSave->Seek(qwSeek, SeekOrigin::Begin);
				}

				// writing
				fsSave->Write(abBuffer, 0, nBytes);
			}
			catch (Exception ^ex) {
				// Failure
				result = false;
			}
			__finally {
				if (fsSave != nullptr) {
					fsSave->Close();
				}
			}

			return result;
		}

		/// <summary>
		/// Read from binary file
		/// </summary>
		bool file::ReadBinary(String ^savefile, __int64 qwSeek, int nBytes, array<Byte> ^abBuffer)
		{
			FileStream^ fsSave = nullptr;
			bool result = true;

			try {
				fsSave = File::Open(savefile, FileMode::Open, FileAccess::Read, FileShare::ReadWrite);
				fsSave->Seek(qwSeek, SeekOrigin::Begin);			// Move to read position
				int nLength = fsSave->Read(abBuffer, 0, nBytes);	// Reading binary data

				if (nLength != nBytes) {
					// Read size mismatch -> failure
					result = false;
				}
			}
			catch (Exception ^ex) {
				// Error occurred
				result = false;
			}
			__finally {
				if (fsSave != nullptr) {
					fsSave->Close();
				}
			}

			return result;
		}

		/// <summary>
		/// Preservation of histogram data
		/// </summary>
		Exception^ file::SaveHistogram(String ^savefile, histogram ^histogram)
		{
			String ^tmpLine = nullptr;
			Exception ^result = nullptr;
			StreamWriter ^swFile = nullptr;

			try {
				try { File::Delete(savefile); }
				catch(...) { /* nothing */ }

				array<array<unsigned int>^>^ auHistogram = histogram->mauHistogram;
				
				// Calculation of summation
				array<__int64> ^aqwTotal = gcnew array<__int64>(auHistogram->GetLength(0));
				Array::Clear(aqwTotal, 0, aqwTotal->Length);

				for (int i = 0; i < aqwTotal->Length; i++) {
					for (int j = 0; j < auHistogram[i]->Length; j++) {
						aqwTotal[i] += (__int64)(auHistogram[i][j]);
					}
				}

				swFile = gcnew StreamWriter(savefile, true, Encoding::Default);

				// save header
				swFile->WriteLine("[system]");
				swFile->WriteLine("save time,{0}", DateTime::Now);

				// save ADC gain
				tmpLine = "ADG";

				for (int i = 0; i < histogram->mauADG->Length; i++) {
					int nAdgMaxCh = constant::Max::GetAdgMaxCh(histogram->mauADG[i]);
					tmpLine += String::Format(",{0}", nAdgMaxCh);
				}

				swFile->WriteLine(tmpLine);

				// outoput total count
				tmpLine = "total count";

				for (int i = 0; i < aqwTotal->Length; i++) {
					tmpLine += ("," + aqwTotal[i].ToString());
				}

				swFile->WriteLine(tmpLine);

				// save histogram data
				swFile->Write(swFile->NewLine);
				swFile->WriteLine("[data]");
				swFile->WriteLine("#ch,CH1,CH2,CH3,CH4");

				for (int i = 0; i < auHistogram[0]->Length; i++) {
					tmpLine = String::Format("{0}", i);

					for (int j = 0; j < auHistogram->Length; j++) {
						tmpLine += String::Format(",{0}", auHistogram[j][i]);
					}

					swFile->WriteLine(tmpLine);
				}
			}
			catch (Exception ^ex) {
				result = ex;
			}
			__finally {
				if (swFile != nullptr) {
					swFile->Close();
				}
			}

			return result;
		}
		
	}
} }


