#include "formMain.h"
#include "TechnoAP-usbmca.h"

using namespace System;
using namespace System::Diagnostics;
using namespace System::Collections::Generic;
using namespace System::ComponentModel;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Text;
using namespace System::Windows::Forms;


namespace usbmcasample {
	
	using namespace FTD2XX_NET;

	/// <summary>
	/// Form load event
	/// </summary>
	System::Void formMain::formMain_Load(System::Object ^sender, System::EventArgs ^e)
	{
		bool blnButton = false;

		// Connection with USB-MCA
		bool bln0th = ConnectDevice(mstrDeviceSerialNumber);

		if (bln0th == true) {
			// Writing measurement parameters
			bool bln1st = WriteConfig();

			if (bln1st == true) {
				// success
				blnButton = true;
			}
			else {
				// error
				MessageBox::Show("device confgi write error...", "error...",
					MessageBoxButtons::OK, MessageBoxIcon::Error);
			}
		}

		btnStart->Enabled = blnButton;
		btnStop->Enabled = blnButton;
	}

	/// <summary>
	/// Device(USB-MCA) connection procedure
	/// </summary>
	bool formMain::ConnectDevice(String ^strSerialNum)
	{
		bool result = true;

		// Connection with USB-MCA by serial number
		FTDI::FT_STATUS ftStatus = mDevice->OpenBySerialNumber(strSerialNum);

		if (ftStatus == FTDI::FT_STATUS::FT_OK) {
			array<FTDI::FT_STATUS> ^astResult = gcnew array<FTDI::FT_STATUS>(8);

			for (int i = 0; i < astResult->Length; i++) {
				astResult[i] = FTDI::FT_STATUS::FT_OK;
			}

			//////////Initialization request for USB-MCA//////////
			int index = 0;
			astResult[index++] = mDevice->ResetDevice();
			astResult[index++] = mDevice->SetBitMode(255, FTDI::FT_BIT_MODES::FT_BIT_MODE_RESET);
			astResult[index++] = mDevice->SetLatency(2);
			astResult[index++] = mDevice->SetTimeouts(100, 100);
			astResult[index++] = mDevice->SetFlowControl(FTDI::FT_FLOW_CONTROL::FT_FLOW_RTS_CTS, 0, 0);
			astResult[index++] = mDevice->Purge(FTDI::FT_PURGE::FT_PURGE_RX | FTDI::FT_PURGE::FT_PURGE_TX);
			System::Threading::Thread::Sleep(500);     // wait accompanying reset
			/////////////////////////////////

			// Display serial number
			txtConnectedDevice->Text = mstrDeviceSerialNumber;

			for (int i = 0; i < astResult->Length; i++) {
				if (astResult[i] != FTDI::FT_STATUS::FT_OK) {
					// There was an error in initial processing
					MessageBox::Show("device initial setting error...", "error...",
						MessageBoxButtons::OK, MessageBoxIcon::Error);
					txtConnectedDevice->Text = String::Empty;
					result = false;
					break;
				}
			}
		}
		else {
			// device open error...
			MessageBox::Show("device initial setting error...", "error...",
				MessageBoxButtons::OK, MessageBoxIcon::Error);
			result = false;
		}

		return result;
	}

	/// <summary>
	/// Measurement parameter write procedure
	/// </summary>
	bool formMain::WriteConfig(void)
	{
		mConfig->mDevice = mDevice;

		array<unsigned int> ^auULD = gcnew array<unsigned int>(mauAdg->Length);

		for (int i = 0; i < mauAdg->Length; i++) {
			mauAdg[i] = (unsigned int)constant::AdcGain::CH_08K;	// 8192ch
			auULD[i] = (unsigned int)constant::Max::GetAdgMaxCh(mauAdg[i]) - 1;
		}

		////////////Writing measurement parameters/////////////////////////
		mConfig->SetADG(mauAdg);             // ADC Gain
		mConfig->SetTHR(80, 80, 80, 80);     // threshold(ch)
		mConfig->SetLLD(95, 95, 95, 95);     // lower level discri(ch)
		mConfig->SetULD(auULD);              // upper level discri(ch)
		mConfig->SetOFS(0, 0, 0, 0);         // offset(ch)
		mConfig->SetPDS(constant::PdsMode::Abs); // absolution
		bool result = mConfig->Write();   // write config
		///////////////////////////////////////////////////////////////////

		return result;
	}

	/// <summary>
	/// Start button click event
	/// </summary>
	System::Void formMain::btnStart_Click(System::Object ^sender, System::EventArgs ^e)
	{
		try {
			mDevice->StopMeasurement();			// Measurement stop
			mDevice->ClearMeasurementData();	// Clear measurement data

			WriteConfig();	// Writing measurement parameters

			mDevice->SetMeasurementTimeMiilliseconds(mqwMeasurementTime);		// Setting measurement time
			mDevice->SetMeasurementDataMode(constant::DataMode::Histogram);		// list mode
			mDevice->SetMeasurementTimeMode(constant::TimeMode::Real);			// Real time mode

			// Start measurement
			mDevice->StartMeasurement();
			tmrMeasure->Start();

			btnStart->Enabled = false;
			mblnSaved = false;
		}
		catch (Exception ^ex) {
			MessageBox::Show(ex->Message, "Exception...",
				MessageBoxButtons::OK, MessageBoxIcon::Error);
		}
	}

	/// <summary>
	/// Stop button click event
	/// </summary>
	System::Void formMain::btnStop_Click(System::Object ^sender, System::EventArgs ^e)
	{
		try {
			tmrMeasure->Stop();				// Stop the measurement timer
			mDevice->StopMeasurement();		// Measurement stop

			if (mblnSaved == false) {
				// Save histogram data at the end of measurement
				data::histogram ^tmpHist = gcnew data::histogram();
				tmpHist->mauADG = mauAdg;
				tmpHist->mauHistogram = mauHistogram;
				common::file::SaveHistogram(mstrSaveFile, tmpHist);
				mblnSaved = true;
			}
		}
		catch (Exception ^ex) {
			MessageBox::Show(ex->Message, "Exception...",
				MessageBoxButtons::OK, MessageBoxIcon::Error);
		}

		btnStart->Enabled = true;
	}

	/// <summary>
	/// Measurement timer event
	/// </summary>
	System::Void formMain::tmrMeasure_Tick(System::Object ^sender, System::EventArgs ^e)
	{
		mStatus = mDevice->ReadStatus();	// Reading measurement status
		__int64 qwElapsedTime = mStatus->getElapsedTimeMillisoconds();	// elapsed time

		for (int i = 0; i < mauHistogram->Length; i++) {	// CH number
			// Reading histogram data
			int maxch = constant::Max::GetAdgMaxCh(mauAdg[i]);
			mauHistogram[i] = mDevice->ReadHistogram((i + 1), maxch);
		}

		if (qwElapsedTime >= mqwMeasurementTime) {
			// Reach measurement time -> stop measurement
			btnStop_Click(sender, e);
		}

		// Display of elapsed time
		txtElapsedTime->Text = qwElapsedTime.ToString();
	}

}

